"use strict";
// Copyright (c) Mito
// Distributed under the terms of the Modified BSD License.
var __importStar = (this && this.__importStar) || function (mod) {
    if (mod && mod.__esModule) return mod;
    var result = {};
    if (mod != null) for (var k in mod) if (Object.hasOwnProperty.call(mod, k)) result[k] = mod[k];
    result["default"] = mod;
    return result;
};
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
const react_1 = __importStar(require("react"));
const Mito_1 = require("./Mito");
const DefaultModal_1 = __importDefault(require("./DefaultModal"));
// import css
require("../../css/merge-modal.css");
const getSharedKeys = (sheetOneIndex, sheetTwoIndex, sheetJSONArray) => {
    // Given two sheets indexes, returns all the columns that are shared between both sheets
    const sheetOneColumnNames = sheetJSONArray[sheetOneIndex].columns;
    const sheetTwoColumnNames = sheetJSONArray[sheetTwoIndex].columns;
    const sharedKeys = sheetOneColumnNames.filter(columnName => sheetTwoColumnNames.includes(columnName)).map(columnHeader => columnHeader.toString());
    return sharedKeys;
};
const getSuggestedKeys = (sheetOneIndex, sheetTwoIndex, sheetJSONArray) => {
    // Given two sheet indexes, reccomends a key to merge on. If there are x shared keys, then:
    // - If x.length == 1, it returns this key as reccomended.
    // - Else, it does not reccomend a key!
    const sharedKeys = getSharedKeys(sheetOneIndex, sheetTwoIndex, sheetJSONArray);
    const sheetOneMergeKey = sharedKeys.length === 1 ? sharedKeys[0] : sheetJSONArray[sheetOneIndex].columns[0];
    const sheetTwoMergeKey = sharedKeys.length === 1 ? sharedKeys[0] : sheetJSONArray[sheetTwoIndex].columns[0];
    return {
        sheetOneMergeKey: sheetOneMergeKey,
        sheetTwoMergeKey: sheetTwoMergeKey
    };
};
class MergeModal extends react_1.default.Component {
    constructor(props) {
        super(props);
        this.updateSheetOneSelection = (e) => {
            const newSheetIndex = parseInt(e.target.value);
            const suggestedKeys = getSuggestedKeys(newSheetIndex, this.state.sheetTwoIndex, this.props.sheetJSONArray);
            this.setState({
                sheetOneIndex: newSheetIndex,
                sheetOneMergeKey: suggestedKeys.sheetOneMergeKey,
                sheetTwoMergeKey: suggestedKeys.sheetTwoMergeKey,
            });
        };
        this.updateSheetOneKeySelection = (e) => {
            const newSheetMergeKey = e.target.value;
            this.setState({ sheetOneMergeKey: newSheetMergeKey });
        };
        this.updateSheetTwoSelection = (e) => {
            const newSheetIndex = parseInt(e.target.value);
            const suggestedKeys = getSuggestedKeys(this.state.sheetOneIndex, newSheetIndex, this.props.sheetJSONArray);
            this.setState({
                sheetTwoIndex: newSheetIndex,
                sheetOneMergeKey: suggestedKeys.sheetOneMergeKey,
                sheetTwoMergeKey: suggestedKeys.sheetTwoMergeKey
            });
        };
        this.updateSheetTwoKeySelection = (e) => {
            const newSheetMergeKey = e.target.value;
            this.setState({ sheetTwoMergeKey: newSheetMergeKey });
        };
        this.completeMerge = () => {
            var _a;
            const selectedColumnsOne = this.props.sheetJSONArray[this.state.sheetOneIndex].columns.filter((columnHeader, index) => {
                if (columnHeader == this.state.sheetOneMergeKey) {
                    return true;
                }
                return this.state.sheetOneSelectedColumns[index];
            });
            const selectedColumnsTwo = this.props.sheetJSONArray[this.state.sheetTwoIndex].columns.filter((columnHeader, index) => {
                if (columnHeader == this.state.sheetTwoMergeKey) {
                    return true;
                }
                return this.state.sheetTwoSelectedColumns[index];
            });
            (_a = window.logger) === null || _a === void 0 ? void 0 : _a.track({
                userId: window.user_id,
                event: 'button_merge_log_event',
                properties: {
                    stage: 'merged',
                    sheet_index_one: this.state.sheetOneIndex,
                    merge_key_one: this.state.sheetOneMergeKey,
                    selected_columns_one: selectedColumnsOne,
                    sheet_index_two: this.state.sheetTwoIndex,
                    merge_key_two: this.state.sheetTwoMergeKey,
                    selected_columns_two: selectedColumnsTwo,
                }
            });
            this.props.send({
                'event': 'edit_event',
                'type': 'merge',
                'id': '123',
                'timestamp': '456',
                'sheet_index_one': this.state.sheetOneIndex,
                'merge_key_one': this.state.sheetOneMergeKey,
                'selected_columns_one': selectedColumnsOne,
                'sheet_index_two': this.state.sheetTwoIndex,
                'merge_key_two': this.state.sheetTwoMergeKey,
                'selected_columns_two': selectedColumnsTwo
            });
            this.props.setModal({ type: Mito_1.ModalEnum.None });
        };
        const sheetOneIndex = 0;
        const sheetTwoIndex = Math.min(1, props.dfNames.length - 1);
        const suggestedKeys = getSuggestedKeys(sheetOneIndex, sheetTwoIndex, props.sheetJSONArray);
        // We default to selecting _all_ columns
        const sheetOneSelectedColumns = this.props.sheetJSONArray[sheetOneIndex].columns.map(() => true);
        const sheetTwoSelectedColumns = this.props.sheetJSONArray[sheetTwoIndex].columns.map(() => true);
        this.state = {
            sheetOneIndex: sheetOneIndex,
            sheetTwoIndex: sheetTwoIndex,
            sheetOneMergeKey: suggestedKeys.sheetOneMergeKey,
            sheetTwoMergeKey: suggestedKeys.sheetTwoMergeKey,
            sheetOneSelectedColumns: sheetOneSelectedColumns,
            sheetTwoSelectedColumns: sheetTwoSelectedColumns,
            sheetOneToggleAll: true,
            sheetTwoToggleAll: true
        };
        this.updateSheetOneSelection = this.updateSheetOneSelection.bind(this);
        this.updateSheetOneKeySelection = this.updateSheetOneKeySelection.bind(this);
        this.updateSheetTwoSelection = this.updateSheetTwoSelection.bind(this);
        this.updateSheetTwoKeySelection = this.updateSheetTwoKeySelection.bind(this);
        this.completeMerge = this.completeMerge.bind(this);
    }
    getColumnSelection(mergeSheetIndex) {
        /*
            Returns the columns selection box, that allows users
            to select which columns they want to keep.
        */
        const sheetIndexName = mergeSheetIndex == 0 ? 'sheetOneIndex' : 'sheetTwoIndex';
        const sheetIndex = mergeSheetIndex == 0 ? this.state.sheetOneIndex : this.state.sheetTwoIndex;
        const mergeKey = mergeSheetIndex == 0 ? this.state.sheetOneMergeKey : this.state.sheetTwoMergeKey;
        const sheetName = this.props.dfNames[sheetIndex];
        return (react_1.default.createElement("div", { className: 'merge-modal-column-selection-container' },
            react_1.default.createElement("div", { className: 'merge-modal-label' },
                "Columns to keep from ",
                sheetName),
            react_1.default.createElement("div", { className: 'merge-modal-column-selection' },
                react_1.default.createElement("div", null,
                    react_1.default.createElement("input", { type: "checkbox", checked: mergeSheetIndex == 0 ? this.state.sheetOneToggleAll : this.state.sheetTwoToggleAll, onChange: () => {
                            if (mergeSheetIndex === 0) {
                                this.setState(prevState => {
                                    const newValue = !prevState.sheetOneToggleAll;
                                    return {
                                        sheetOneToggleAll: newValue,
                                        sheetOneSelectedColumns: prevState.sheetOneSelectedColumns.map(() => newValue)
                                    };
                                });
                            }
                            else {
                                this.setState(prevState => {
                                    const newValue = !prevState.sheetTwoToggleAll;
                                    return {
                                        sheetTwoToggleAll: newValue,
                                        sheetTwoSelectedColumns: prevState.sheetTwoSelectedColumns.map(() => newValue)
                                    };
                                });
                            }
                        }, className: "form-check-input" }),
                    "Select all"),
                this.props.sheetJSONArray[this.state[sheetIndexName]].columns.map((columnHeader, index) => {
                    // We don't display the merge key!
                    if (mergeKey == columnHeader) {
                        return undefined;
                    }
                    const isSelected = mergeSheetIndex == 0 ?
                        this.state.sheetOneSelectedColumns[index] :
                        this.state.sheetTwoSelectedColumns[index];
                    return (react_1.default.createElement("div", { key: columnHeader },
                        react_1.default.createElement("input", { key: columnHeader, type: "checkbox", name: columnHeader.toString(), checked: isSelected, onChange: () => {
                                if (mergeSheetIndex === 0) {
                                    this.setState(prevState => {
                                        const newSheetOneSelectedColumns = [...prevState.sheetOneSelectedColumns];
                                        newSheetOneSelectedColumns[index] = !newSheetOneSelectedColumns[index];
                                        return {
                                            sheetOneSelectedColumns: newSheetOneSelectedColumns
                                        };
                                    });
                                }
                                else {
                                    this.setState(prevState => {
                                        const newSheetTwoSelectedColumns = [...prevState.sheetTwoSelectedColumns];
                                        newSheetTwoSelectedColumns[index] = !newSheetTwoSelectedColumns[index];
                                        return {
                                            sheetTwoSelectedColumns: newSheetTwoSelectedColumns
                                        };
                                    });
                                }
                            }, className: "form-check-input" }),
                        columnHeader));
                }))));
    }
    render() {
        const sheetOneSelect = (react_1.default.createElement("select", { className: 'merge-modal-sheet-dropdown', onChange: this.updateSheetOneSelection }, this.props.dfNames.map((dfName, index) => {
            return (react_1.default.createElement("option", { key: dfName, value: index, selected: this.state.sheetOneIndex === index }, dfName));
        })));
        const sheetOneMergeKeySelect = (react_1.default.createElement("select", { className: 'merge-modal-key-dropdown', onChange: this.updateSheetOneKeySelection }, this.props.sheetJSONArray[this.state.sheetOneIndex].columns.map((columnHeader) => {
            return (react_1.default.createElement("option", { key: columnHeader, value: columnHeader, selected: this.state.sheetOneMergeKey === columnHeader }, columnHeader));
        })));
        const sheetOneColumnsToKeep = this.getColumnSelection(0);
        const sheetTwoSelect = (react_1.default.createElement("select", { className: 'merge-modal-sheet-dropdown', onChange: this.updateSheetTwoSelection }, this.props.dfNames.map((dfName, index) => {
            return (react_1.default.createElement("option", { key: dfName, value: index, selected: this.state.sheetTwoIndex === index }, dfName));
        })));
        const sheetTwoMergeKeySelect = (react_1.default.createElement("select", { className: 'merge-modal-key-dropdown', onChange: this.updateSheetTwoKeySelection }, this.props.sheetJSONArray[this.state.sheetTwoIndex].columns.map((columnHeader) => {
            return (react_1.default.createElement("option", { key: columnHeader, value: columnHeader, selected: this.state.sheetTwoMergeKey === columnHeader }, columnHeader));
        })));
        const sheetTwoColumnsToKeep = this.getColumnSelection(1);
        // if there are no sheets to merge, don't try to display modal
        if (this.props.dfNames.length === 0) {
            return (react_1.default.createElement(DefaultModal_1.default, { header: 'Merge two sheets', modalType: Mito_1.ModalEnum.Merge, viewComponent: react_1.default.createElement(react_1.Fragment, null, "There are no sheets to merge"), buttons: react_1.default.createElement(react_1.Fragment, null,
                    react_1.default.createElement("div", { className: 'modal-close-button', onClick: () => { this.props.setModal({ type: Mito_1.ModalEnum.None }); } }, " Close ")) }));
        }
        return (react_1.default.createElement(DefaultModal_1.default, { header: 'Merge two sheets', modalType: Mito_1.ModalEnum.Merge, viewComponent: react_1.default.createElement(react_1.Fragment, null,
                react_1.default.createElement("div", null,
                    react_1.default.createElement("div", { className: 'merge-modal-sheet-and-key' },
                        react_1.default.createElement("div", { className: 'merge-modal-sheet' },
                            react_1.default.createElement("p", { className: 'merge-modal-label' }, "First Sheet"),
                            sheetOneSelect),
                        react_1.default.createElement("div", { className: 'merge-modal-key' },
                            react_1.default.createElement("p", { className: 'merge-modal-label' }, "Merge Key"),
                            sheetOneMergeKeySelect)),
                    sheetOneColumnsToKeep,
                    react_1.default.createElement("div", { className: 'merge-modal-sheet-and-key' },
                        react_1.default.createElement("div", { className: 'merge-modal-sheet' },
                            react_1.default.createElement("p", { className: 'merge-modal-label' }, "Second Sheet"),
                            sheetTwoSelect),
                        react_1.default.createElement("div", { className: 'merge-modal-key' },
                            react_1.default.createElement("p", { className: 'merge-modal-label' }, "Merge Key"),
                            sheetTwoMergeKeySelect)),
                    sheetTwoColumnsToKeep)), buttons: react_1.default.createElement(react_1.Fragment, null,
                react_1.default.createElement("div", { className: 'modal-close-button modal-dual-button-left', onClick: () => { this.props.setModal({ type: Mito_1.ModalEnum.None }); } }, " Close "),
                react_1.default.createElement("div", { className: 'modal-action-button modal-dual-button-right', onClick: this.completeMerge },
                    " ",
                    "Merge")) }));
    }
}
exports.default = MergeModal;
//# sourceMappingURL=MergeModal.js.map