"use strict";
// Copyright (c) Mito
// Distributed under the terms of the Modified BSD License.
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
const react_1 = __importDefault(require("react"));
const ag_grid_react_1 = require("ag-grid-react");
require("ag-grid-community/dist/styles/ag-grid.css");
require("ag-grid-community/dist/styles/ag-theme-alpine.css");
require("../../css/mitosheet.css");
const MitoCellEditor_1 = __importDefault(require("./MitoCellEditor"));
const MitoCellEditor_2 = require("./MitoCellEditor");
// And functions for building components
const gridData_1 = require("../utils/gridData");
const MitoSheet = (props) => {
    function onGridReady(params) {
        if (window.gridApiMap === undefined) {
            window.gridApiMap = new Map();
        }
        window.gridApiMap.set(props.model_id, params.api);
    }
    function onGridColumnsChanged(params) {
        /*
            Ag-grid does some sort of caching that means if we rename a column
            it will treat this as a new column that got added to the _end_.

            This function runs whenever the columns change.
            
            It makes sure that the columns are in the correct order. If they aren't,
            it then places them in the correct order.
        */
        const columnState = params.columnApi.getColumnState();
        const correctColumnOrder = props.sheetJSON.columns;
        for (let i = 0; i < correctColumnOrder.length; i++) {
            const columnName = correctColumnOrder[i].toString();
            // If, ignoring the index column, this column is not in the right place, then
            // we have some reordering to do!
            const correctIndex = i + 1;
            const currentDisplayIndex = columnState.findIndex((columnState) => {
                return columnState.colId === columnName;
            });
            if (correctIndex !== currentDisplayIndex) {
                params.columnApi.moveColumn(columnName, correctIndex);
            }
        }
    }
    const cellValueChanged = (e) => {
        const column = e.colDef.field ? e.colDef.field : "";
        const newValue = e.newValue;
        props.sendCellValueUpdate(column, newValue);
    };
    const columns = gridData_1.buildGridColumns(props.sheetJSON.columns, props.columnSpreadsheetCodeJSON, props.columnFiltersJSON, props.formulaBarValue, props.editingColumn, props.cursorIndex, props.setEditingMode, props.setEditingFormula, props.setCursorIndex, props.setModal);
    const rowData = gridData_1.buildGridData(props.sheetJSON);
    const frameworkComponents = {
        simpleEditor: MitoCellEditor_1.default,
    };
    return (react_1.default.createElement("div", null,
        react_1.default.createElement("div", { className: "ag-theme-alpine ag-grid" },
            react_1.default.createElement(ag_grid_react_1.AgGridReact, { onGridReady: onGridReady, onGridColumnsChanged: onGridColumnsChanged, onCellFocused: (e) => props.cellFocused(e), onColumnMoved: (e) => props.columnMoved(e), onDragStopped: () => props.columnDragStopped(), rowData: rowData, frameworkComponents: frameworkComponents, suppressKeyboardEvent: (params) => {
                    /*
                        While we're editing a cell, we suppress events that we use
                        to do things within the editor.

                        NOTE: this function should suppress the events matched in onKeyDown
                        in MitoCellEditor!
                    */
                    if (!params.editing) {
                        return false;
                    }
                    return MitoCellEditor_2.TAKE_SUGGESTION_KEYS.includes(params.event.key) ||
                        MitoCellEditor_2.ARROW_UP_KEYS.includes(params.event.key) ||
                        MitoCellEditor_2.ARROW_DOWN_KEYS.includes(params.event.key);
                }, onCellValueChanged: cellValueChanged, suppressDragLeaveHidesColumns: true, suppressColumnMoveAnimation: true }, columns))));
};
exports.default = MitoSheet;
//# sourceMappingURL=MitoSheet.js.map