"use strict";
// Copyright (c) Mito
// Distributed under the terms of the Modified BSD License.
var __importStar = (this && this.__importStar) || function (mod) {
    if (mod && mod.__esModule) return mod;
    var result = {};
    if (mod != null) for (var k in mod) if (Object.hasOwnProperty.call(mod, k)) result[k] = mod[k];
    result["default"] = mod;
    return result;
};
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
const react_1 = __importStar(require("react"));
const Mito_1 = require("../Mito");
const DefaultModal_1 = __importDefault(require("../DefaultModal"));
// import css
require("../../../css/margins.css");
require("../../../css/filter_modal.css");
var SharedFilterCondition;
(function (SharedFilterCondition) {
    SharedFilterCondition["EMPTY"] = "empty";
    SharedFilterCondition["NOT_EMPTY"] = "not_empty";
})(SharedFilterCondition = exports.SharedFilterCondition || (exports.SharedFilterCondition = {}));
var StringFilterCondition;
(function (StringFilterCondition) {
    StringFilterCondition["CONTAINS"] = "contains";
    StringFilterCondition["STRING_EXACTLY"] = "string_exactly";
})(StringFilterCondition = exports.StringFilterCondition || (exports.StringFilterCondition = {}));
var NumberFilterCondition;
(function (NumberFilterCondition) {
    NumberFilterCondition["NUMBER_EXACTLY"] = "number_exactly";
    NumberFilterCondition["GREATER"] = "greater";
    NumberFilterCondition["GREATER_THAN_OR_EQUAL"] = "greater_than_or_equal";
    NumberFilterCondition["LESS"] = "less";
    NumberFilterCondition["LESS_THAN_OR_EQUAL"] = "less_than_or_equal";
})(NumberFilterCondition = exports.NumberFilterCondition || (exports.NumberFilterCondition = {}));
var DateFilterCondition;
(function (DateFilterCondition) {
    DateFilterCondition["DATETIME_EXTACTLY"] = "datetime_exactly";
    DateFilterCondition["DATETIME_GREATER_THAN"] = "datetime_greater";
    DateFilterCondition["DATETIME_GREATER_THAN_OR_EQUAL"] = "datetime_greater_than_or_equal";
    DateFilterCondition["DATETIME_LESS"] = "datetime_less";
    DateFilterCondition["DATETIME_LESS_THAN_OR_EQUAL"] = "datetime_less_than_or_equal";
})(DateFilterCondition = exports.DateFilterCondition || (exports.DateFilterCondition = {}));
// if one of these filters are selected, don't display the value input
const noInputFilters = [SharedFilterCondition.EMPTY, SharedFilterCondition.NOT_EMPTY];
const getNumberDropdownOptions = () => {
    return (react_1.default.createElement(react_1.default.Fragment, null,
        react_1.default.createElement("option", { value: NumberFilterCondition.NUMBER_EXACTLY }, "="),
        react_1.default.createElement("option", { value: NumberFilterCondition.GREATER }, ">"),
        react_1.default.createElement("option", { value: NumberFilterCondition.GREATER_THAN_OR_EQUAL }, "\u2265"),
        react_1.default.createElement("option", { value: NumberFilterCondition.LESS }, "<"),
        react_1.default.createElement("option", { value: NumberFilterCondition.LESS_THAN_OR_EQUAL }, "\u2264")));
};
const getStringDropdownOptions = () => {
    return (react_1.default.createElement(react_1.default.Fragment, null,
        react_1.default.createElement("option", { value: StringFilterCondition.CONTAINS }, "contains"),
        react_1.default.createElement("option", { value: StringFilterCondition.STRING_EXACTLY }, "is exactly")));
};
const getDatetimeDropdownOptions = () => {
    return (react_1.default.createElement(react_1.default.Fragment, null,
        react_1.default.createElement("option", { value: DateFilterCondition.DATETIME_EXTACTLY }, "="),
        react_1.default.createElement("option", { value: DateFilterCondition.DATETIME_GREATER_THAN }, ">"),
        react_1.default.createElement("option", { value: DateFilterCondition.DATETIME_GREATER_THAN_OR_EQUAL }, "\u2265"),
        react_1.default.createElement("option", { value: DateFilterCondition.DATETIME_LESS }, "<"),
        react_1.default.createElement("option", { value: DateFilterCondition.DATETIME_LESS_THAN_OR_EQUAL }, "\u2264")));
};
const getSharedDropdownOptions = () => {
    return (react_1.default.createElement(react_1.default.Fragment, null,
        react_1.default.createElement("option", { value: SharedFilterCondition.EMPTY }, "is empty"),
        react_1.default.createElement("option", { value: SharedFilterCondition.NOT_EMPTY }, "is not empty")));
};
const getNewEmptyFilter = (columnType) => {
    switch (columnType) {
        case 'string':
            return {
                type: 'string',
                condition: StringFilterCondition.CONTAINS,
                value: ''
            };
        case 'number':
            return {
                type: 'number',
                condition: NumberFilterCondition.NUMBER_EXACTLY,
                value: 0
            };
        case 'datetime':
            return {
                type: 'datetime',
                condition: DateFilterCondition.DATETIME_EXTACTLY,
                value: ''
            };
    }
};
/*
    A modal that allows a user to filter a column
*/
class FilterModal extends react_1.default.Component {
    constructor(props) {
        super(props);
        this.filterColumn = () => {
            var _a;
            // We make sure all the filter values are defined, as serialized
            // JSON messages loose the undefined values
            const filters = this.state.filters.map(filter => {
                return {
                    'type': filter.type,
                    'condition': filter.condition,
                    'value': filter.value === undefined ? 0 : filter.value
                };
            });
            (_a = window.logger) === null || _a === void 0 ? void 0 : _a.track({
                userId: window.user_id,
                event: 'button_filter_log_event',
                properties: {
                    'sheet_index': this.props.selectedSheetIndex,
                    'column_header': this.props.columnHeader,
                    'filters': filters,
                }
            });
            this.props.send({
                'event': 'edit_event',
                'type': 'filter',
                'sheet_index': this.props.selectedSheetIndex,
                'column_header': this.props.columnHeader,
                'operator': this.state.operator,
                'filters': filters,
                'id': '123',
                'timestamp': '456'
            });
            this.props.setModal({ type: Mito_1.ModalEnum.None });
        };
        this.state = {
            filters: this.props.filters,
            operator: this.props.operator
        };
        this.filterColumn = this.filterColumn.bind(this);
        this.buildFilters = this.buildFilters.bind(this);
    }
    buildFilters() {
        /*
            Builds all the JSX for the filter selections, for all of the filters in
            this.state.filters
        */
        const handleToggleOperator = (e) => {
            // Occurs when the user switches which operator is being used
            const newOperator = e.target.value;
            this.setState({
                operator: newOperator
            });
        };
        const handleConditionChange = (filterIndex, e) => {
            // Occurs when a user changes the condition on a specific filter
            // This case is safe, as users can only select from a list of the enum
            const newCondition = e.target.value;
            this.setState(prevState => {
                const newFilters = [...prevState.filters];
                // Note: we save the new condition outside the setState, as react does
                // not keep events around! 
                newFilters[filterIndex].condition = newCondition;
                if (noInputFilters.includes(newCondition)) {
                    // Clear the value, if this condition has no value
                    newFilters[filterIndex].value = undefined;
                }
                return {
                    filters: newFilters,
                };
            });
        };
        const handleValueChange = (filterIndex, e) => {
            // Occurs when the user changes a value on a specific filter, which is then
            // saved to the state for this filter
            const filter = this.state.filters[filterIndex];
            let newValue = e.target.value;
            // Cast it to the correct type
            if (filter.type === 'number') {
                const float = parseFloat(newValue);
                if (!Number.isNaN(float)) {
                    newValue = float;
                }
            }
            this.setState(prevState => {
                const newFilters = [...prevState.filters];
                // Note: we save the new condition outside the setState, as react does
                // not keep events around! 
                newFilters[filterIndex].value = newValue;
                return {
                    filters: newFilters,
                };
            });
        };
        const removeFilter = (filterIndex) => {
            // Removes the filter at the given filterIndex 
            this.setState(prevState => {
                const newFilters = [...prevState.filters];
                newFilters.splice(filterIndex, 1);
                return {
                    filters: newFilters
                };
            });
        };
        const getFilter = (filter, filterIndex) => {
            // Builds the table row element for a given filter at a given index
            var _a;
            // We use styles to hide divs we don't want, to keep the spacing the same
            const operatorStyle = filterIndex === 0 ? { visibility: 'hidden' } : { visibility: 'visible' };
            const inputStyle = noInputFilters.includes(filter.condition) ? { visibility: 'hidden' } : { visibility: 'visible' };
            return (react_1.default.createElement(react_1.default.Fragment, null,
                react_1.default.createElement("tr", null,
                    react_1.default.createElement("td", null,
                        react_1.default.createElement("select", { className: 'select filter-modal-input', style: operatorStyle, value: this.state.operator, onChange: handleToggleOperator },
                            react_1.default.createElement("option", { value: 'And' }, "And"),
                            react_1.default.createElement("option", { value: 'Or' }, "Or"))),
                    react_1.default.createElement("td", null,
                        react_1.default.createElement("select", { className: "select filter-modal-condition-select", value: filter.condition, onChange: e => handleConditionChange(filterIndex, e) },
                            filter.type === 'number' && getNumberDropdownOptions(),
                            filter.type === 'string' && getStringDropdownOptions(),
                            filter.type === 'datetime' && getDatetimeDropdownOptions(),
                            getSharedDropdownOptions())),
                    react_1.default.createElement("td", null,
                        react_1.default.createElement("input", { className: "input filter-modal-value-input", style: inputStyle, value: (_a = filter.value) === null || _a === void 0 ? void 0 : _a.toString(), onChange: e => handleValueChange(filterIndex, e) })),
                    react_1.default.createElement("td", null,
                        react_1.default.createElement("div", { className: 'ml-1 mr-1', onClick: () => removeFilter(filterIndex) },
                            react_1.default.createElement("svg", { width: "13", height: "3", viewBox: "0 0 13 3", fill: "none", xmlns: "http://www.w3.org/2000/svg" },
                                react_1.default.createElement("rect", { width: "13", height: "3", rx: "1", fill: "#B1B1B1" })))))));
        };
        return (react_1.default.createElement("table", null,
            this.state.filters.length > 0 &&
                react_1.default.createElement("tr", null,
                    react_1.default.createElement("td", null),
                    react_1.default.createElement("td", null, "Filter Condition"),
                    react_1.default.createElement("td", null, "Value")),
            this.state.filters.map((filter, filterIndex) => {
                return getFilter(filter, filterIndex);
            })));
    }
    render() {
        const addFilter = () => {
            this.setState(prevState => {
                const newFilters = [...prevState.filters];
                newFilters.push(getNewEmptyFilter(this.props.columnType));
                return {
                    filters: newFilters
                };
            });
        };
        return (react_1.default.createElement(DefaultModal_1.default, { header: `Filter Column: ${this.props.columnHeader}`, modalType: Mito_1.ModalEnum.Filter, viewComponent: react_1.default.createElement(react_1.Fragment, null,
                react_1.default.createElement("div", { className: "filter-modal-container" },
                    react_1.default.createElement("div", { className: "filter-modal-center" },
                        this.state.filters.length === 0 &&
                            react_1.default.createElement("p", { className: 'mb-1' }, "Hit the Add Filter button below to begin filtering this column."),
                        this.buildFilters(),
                        react_1.default.createElement("div", { className: 'filter-modal-add-value', onClick: addFilter }, "+ Add Filter")))), buttons: react_1.default.createElement(react_1.Fragment, null,
                react_1.default.createElement("div", { className: 'modal-close-button modal-dual-button-left', onClick: () => { this.props.setModal({ type: Mito_1.ModalEnum.None }); } }, " Close "),
                react_1.default.createElement("div", { className: 'modal-action-button modal-dual-button-right', onClick: this.filterColumn },
                    " ",
                    "Filter")) }));
    }
}
exports.default = FilterModal;
//# sourceMappingURL=FilterModal.js.map