"use strict";
// Copyright (c) Mito
// Distributed under the terms of the Modified BSD License.
var __importStar = (this && this.__importStar) || function (mod) {
    if (mod && mod.__esModule) return mod;
    var result = {};
    if (mod != null) for (var k in mod) if (Object.hasOwnProperty.call(mod, k)) result[k] = mod[k];
    result["default"] = mod;
    return result;
};
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
const react_1 = __importStar(require("react"));
const Mito_1 = require("../Mito");
const DefaultModal_1 = __importDefault(require("../DefaultModal"));
require("../../../css/group-modal.css");
// The aggregation functions we're supporting, out of the box
const AGGREGATION_OPTIONS = [
    'sum', 'mean', 'min', 'max', 'count'
];
var KeyType;
(function (KeyType) {
    KeyType["Row"] = "Row";
    KeyType["Col"] = "Col";
})(KeyType = exports.KeyType || (exports.KeyType = {}));
class GroupModal extends react_1.default.Component {
    constructor(props) {
        super(props);
        this.completeGroup = () => {
            var _a;
            /*
                Completes the group operation by sending information for the group
                to the backend.
    
                Errors if no group-by keys are selected.
            */
            const groupByRows = this.props.sheetJSONArray[this.props.selectedSheetIndex].columns.filter((columnHeader, index) => {
                return this.state.selectedGroupByRows[index];
            });
            const groupByCols = this.props.sheetJSONArray[this.props.selectedSheetIndex].columns.filter((columnHeader, index) => {
                return this.state.selectedGroupByCols[index];
            });
            // If there are no keys grouped by, we insert an error, and do not let the user submit
            if (groupByRows.length == 0 && groupByCols.length == 0) {
                this.setState({
                    keyError: 'Please select a at least one row or column before submitting'
                });
                return;
            }
            // Build the values mapping
            const values = {};
            this.state.values.forEach((value) => {
                values[value[0]] = value[1];
            });
            // Log
            (_a = window.logger) === null || _a === void 0 ? void 0 : _a.track({
                userId: window.user_id,
                event: 'button_group_log_event',
                properties: {
                    sheet_index: this.props.selectedSheetIndex,
                    group_by_rows: groupByRows,
                    group_by_cols: groupByCols,
                    values: values
                }
            });
            this.props.send({
                'event': 'edit_event',
                'type': 'group_edit',
                'id': '123',
                'timestamp': '456',
                sheet_index: this.props.selectedSheetIndex,
                group_by_rows: groupByRows,
                group_by_columns: groupByCols,
                values: values
            });
            this.props.setModal({ type: Mito_1.ModalEnum.None });
        };
        const columns = this.props.sheetJSONArray[props.selectedSheetIndex].columns;
        // We default to selecting _no_ columns to group by
        const selectedGroupByRows = columns.map(() => false);
        const selectedGroupByCols = columns.map(() => false);
        // If there are any columns, we select the first one as the value, and just sum it
        // NOTE: this is because users don't notice they can do this when we don't show them
        const values = [];
        if (columns.length > 0) {
            values.push([columns[0], AGGREGATION_OPTIONS[0]]);
        }
        this.state = {
            selectedGroupByRows: selectedGroupByRows,
            selectedGroupByCols: selectedGroupByCols,
            values: values,
            keyError: ''
        };
        this.getGroupByKeys = this.getGroupByKeys.bind(this);
        this.getValues = this.getValues.bind(this);
        this.addValue = this.addValue.bind(this);
        this.completeGroup = this.completeGroup.bind(this);
    }
    getGroupByKeys(keyType) {
        /*
            Returns the group key selection boxes in a container that
            allow them to scroll and easily be selected.
        */
        return (
        // add a top margin to the column selection div
        react_1.default.createElement("div", { className: keyType === KeyType.Col ? 'group-modal-column-selection-container' : '' },
            react_1.default.createElement("div", { className: 'group-modal-label' }, keyType === KeyType.Row ? 'Rows' : 'Columns'),
            react_1.default.createElement("div", { className: 'group-modal-column-selection' }, this.props.sheetJSONArray[this.props.selectedSheetIndex].columns.map((columnHeader, index) => {
                // determine if the key is selected
                const isSelected = keyType === KeyType.Row ? this.state.selectedGroupByRows[index] : this.state.selectedGroupByCols[index];
                return (react_1.default.createElement("div", { key: columnHeader },
                    react_1.default.createElement("input", { key: columnHeader, type: "checkbox", name: columnHeader.toString(), checked: isSelected, onChange: () => {
                            if (keyType === KeyType.Row) {
                                this.setState(prevState => {
                                    const newSelectedGroupByRows = [...prevState.selectedGroupByRows];
                                    newSelectedGroupByRows[index] = !newSelectedGroupByRows[index];
                                    return {
                                        selectedGroupByRows: newSelectedGroupByRows,
                                        keyError: '' // Clear the key error
                                    };
                                });
                            }
                            else {
                                this.setState(prevState => {
                                    const newSelectedGroupByCols = [...prevState.selectedGroupByCols];
                                    newSelectedGroupByCols[index] = !newSelectedGroupByCols[index];
                                    return {
                                        selectedGroupByCols: newSelectedGroupByCols,
                                        keyError: '' // Clear the key error
                                    };
                                });
                            }
                        }, className: "form-check-input" }),
                    columnHeader));
            }))));
    }
    getValues() {
        /*
            Returns the values JSX element, which are tuples of columnHeader, aggregation
            function selection boxes.
        */
        return (react_1.default.createElement("div", { className: 'mt-1' },
            react_1.default.createElement("div", { className: 'group-modal-value-mapping' },
                react_1.default.createElement("div", { className: 'group-modal-label group-modal-value-mapping-key' }, "Value"),
                react_1.default.createElement("div", { className: 'group-modal-label' }, "Aggregation Method")),
            react_1.default.createElement("div", { className: 'group-modal-value-mapping-container' }, this.state.values.map((value, index) => {
                const columnHeader = value[0];
                const aggregationOption = value[1];
                const changeColumnSelect = (e) => {
                    // Save outside the set state, because react cleans up events, 
                    // so it may be undefined inside setState callback
                    const columnHeader_ = e.target.value;
                    this.setState(prevState => {
                        const newValues = [...prevState.values];
                        newValues[index][0] = columnHeader_;
                        return {
                            values: newValues
                        };
                    });
                };
                const changeAggSelect = (e) => {
                    // Save outside the set state, because react cleans up events, 
                    // so it may be undefined inside setState callback
                    const aggFunction = e.target.value;
                    this.setState(prevState => {
                        const newValues = [...prevState.values];
                        newValues[index][1] = aggFunction;
                        return {
                            values: newValues
                        };
                    });
                };
                const removeValueMapping = () => {
                    this.setState(prevState => {
                        const newValues = [...prevState.values];
                        newValues.splice(index, 1);
                        return {
                            values: newValues
                        };
                    });
                };
                const columnSelect = (react_1.default.createElement("select", { className: 'group-modal-dropdown group-modal-value-mapping-key', onChange: changeColumnSelect }, this.props.sheetJSONArray[this.props.selectedSheetIndex].columns.map((columnHeader_) => {
                    return (react_1.default.createElement("option", { key: columnHeader_, value: columnHeader_, selected: columnHeader === columnHeader_ }, columnHeader_));
                })));
                const aggregationSelect = (react_1.default.createElement("select", { className: 'group-modal-dropdown group-modal-value-mapping-value', onChange: changeAggSelect }, AGGREGATION_OPTIONS.map(aggregationOption_ => {
                    return (react_1.default.createElement("option", { key: aggregationOption_, value: aggregationOption_, selected: aggregationOption_ === aggregationOption }, aggregationOption_));
                })));
                return (react_1.default.createElement("div", { key: index, className: 'group-modal-value-mapping mb-1' },
                    columnSelect,
                    aggregationSelect,
                    react_1.default.createElement("div", { className: 'ml-1 mr-1', onClick: removeValueMapping },
                        react_1.default.createElement("svg", { width: "13", height: "3", viewBox: "0 0 13 3", fill: "none", xmlns: "http://www.w3.org/2000/svg" },
                            react_1.default.createElement("rect", { width: "13", height: "3", rx: "1", fill: "#B1B1B1" })))));
            }))));
    }
    addValue() {
        /*
            Adds a new value (columnHeader, aggregationFunction) mapping to this.state.values.

            Has some heuristics to guess which columnHeader you want!
        */
        this.setState(prevState => {
            const newValues = [...prevState.values];
            // We get the first column header in the array that 
            const columns = this.props.sheetJSONArray[this.props.selectedSheetIndex].columns;
            // If there is no columns in this, then we have nothing to group!
            if (columns.length === 0) {
                newValues.push(['none', AGGREGATION_OPTIONS[0]]);
                return {
                    values: newValues
                };
            }
            // Find the first column that has not been selected as a key, and
            // also has not been grouped too, as a heuristic for what the user
            // wants to aggregate
            const unaggregatedColumns = columns.filter((columnHeader, index) => {
                return !prevState.selectedGroupByRows[index] && prevState.values.findIndex((value) => {
                    return value[0] == columnHeader;
                }) < 0;
            });
            // Take the first unaggregated column if it exists, otherwise just take the 
            // first column
            const newColumn = unaggregatedColumns.length > 0 ? unaggregatedColumns[0] : columns[0];
            newValues.push([newColumn, AGGREGATION_OPTIONS[0]]);
            return {
                values: newValues
            };
        });
    }
    render() {
        const groupByRows = this.getGroupByKeys(KeyType.Row);
        const groupByCols = this.getGroupByKeys(KeyType.Col);
        const values = this.getValues();
        // if there are no sheets to group, don't try to display modal
        if (this.props.dfNames.length === 0) {
            return (react_1.default.createElement(DefaultModal_1.default, { header: 'Group ' + this.props.dfNames[this.props.selectedSheetIndex], modalType: Mito_1.ModalEnum.Merge, viewComponent: react_1.default.createElement(react_1.Fragment, null, "There are no sheets to group"), buttons: react_1.default.createElement(react_1.Fragment, null,
                    react_1.default.createElement("div", { className: 'modal-close-button', onClick: () => { this.props.setModal({ type: Mito_1.ModalEnum.None }); } }, " Close ")) }));
        }
        return (react_1.default.createElement(DefaultModal_1.default, { header: 'Group ' + this.props.dfNames[this.props.selectedSheetIndex], modalType: Mito_1.ModalEnum.Merge, viewComponent: react_1.default.createElement(react_1.Fragment, null,
                react_1.default.createElement("div", null,
                    groupByRows,
                    groupByCols,
                    react_1.default.createElement("div", { className: 'group-modal-key-error' }, this.state.keyError),
                    values,
                    react_1.default.createElement("div", { className: 'group-modal-add-value', onClick: this.addValue }, "+ Add Value"))), buttons: react_1.default.createElement(react_1.Fragment, null,
                react_1.default.createElement("div", { className: 'modal-close-button modal-dual-button-left', onClick: () => { this.props.setModal({ type: Mito_1.ModalEnum.None }); } }, " Close "),
                react_1.default.createElement("div", { className: 'modal-action-button modal-dual-button-right', onClick: this.completeGroup },
                    " ",
                    "Group")) }));
    }
}
exports.default = GroupModal;
//# sourceMappingURL=GroupModal.js.map