"use strict";
// Copyright (c) Mito
// Distributed under the terms of the Modified BSD License.
var __importStar = (this && this.__importStar) || function (mod) {
    if (mod && mod.__esModule) return mod;
    var result = {};
    if (mod != null) for (var k in mod) if (Object.hasOwnProperty.call(mod, k)) result[k] = mod[k];
    result["default"] = mod;
    return result;
};
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
const react_1 = __importStar(require("react"));
const Mito_1 = require("../Mito");
const DefaultModal_1 = __importDefault(require("../DefaultModal"));
// import css
require("../../../css/save-analysis-modal.css");
// A set of characters we don't allow in analysis names,
// just to avoid file name issues
// Taken from: https://www.mtu.edu/umc/services/digital/writing/characters-avoid/
const ILLEGAL_FILENAME_CHARACTERS = [
    '#',
    '%',
    '&',
    '{', '}',
    '/', '\\',
    '<', '>',
    '*',
    '?',
    '$',
    '!',
    '\'', '"',
    ':',
    '@'
];
function getAnalysisNameError(analysisName) {
    /*
        Helper function for validating named analyses. Avoids a set of illegal characters.

        Returns the empty string if there is no error.
    */
    if (analysisName.length == 0) {
        return 'Please enter a name for your analysis';
    }
    for (let i = 0; i < ILLEGAL_FILENAME_CHARACTERS.length; i++) {
        if (analysisName.indexOf(ILLEGAL_FILENAME_CHARACTERS[i]) > -1) {
            return `Invalid analysis name. Please remove the "${ILLEGAL_FILENAME_CHARACTERS[i]}"`;
        }
    }
    return '';
}
/*
  A modal that allows a user to save an analysis with the input
  name.
*/
const SaveAnalysis = (props) => {
    const [saveAnalysisName, setSaveAnalysisName] = react_1.useState(props.saveAnalysisName !== undefined ? props.saveAnalysisName : '');
    const [analysisNameError, setAnalysisNameError] = react_1.useState('');
    const clickSave = () => {
        var _a;
        // Don't let the user submit if they have an invalid analysis name!
        if (getAnalysisNameError(saveAnalysisName) !== '') {
            setAnalysisNameError(`Please fix before saving: ${getAnalysisNameError(saveAnalysisName)}`);
            return;
        }
        if (props.savedAnalysisNames.indexOf(saveAnalysisName) > -1) {
            // If user is overwriting an analysis, we display a popup to let them know. 
            props.setModal({
                type: Mito_1.ModalEnum.SaveAnalysisOverwrite,
                saveAnalysisName: saveAnalysisName
            });
        }
        else {
            // Otherwise, we just log and save the analysis
            (_a = window.logger) === null || _a === void 0 ? void 0 : _a.track({
                userId: window.user_id,
                event: 'button_save_analysis_log_event',
                properties: {
                    analysis_name: saveAnalysisName
                }
            });
            props.send({
                'event': 'update_event',
                'type': 'save_analysis',
                'analysis_name': saveAnalysisName
            });
            props.setModal({ type: Mito_1.ModalEnum.None });
        }
    };
    const onChangeAnalysisName = (newAnalysisName) => {
        setSaveAnalysisName(newAnalysisName);
        setAnalysisNameError(getAnalysisNameError(newAnalysisName));
    };
    return (react_1.default.createElement(DefaultModal_1.default, { header: `Save your current analysis`, modalType: Mito_1.ModalEnum.ReplayAnalysis, viewComponent: react_1.default.createElement(react_1.Fragment, null,
            react_1.default.createElement("div", null,
                react_1.default.createElement("div", null,
                    react_1.default.createElement("div", { className: 'save-analysis-label' }, "Name your analysis"),
                    react_1.default.createElement("input", { className: "modal-input", type: "text", placeholder: 'Monthly Sales Analysis', value: saveAnalysisName, onChange: (e) => onChangeAnalysisName(e.target.value) }),
                    analysisNameError !== '' &&
                        react_1.default.createElement("p", { className: 'save-analysis-name-error' }, analysisNameError)),
                react_1.default.createElement("div", { className: 'mt-2 save-analysis-analysis-selection-container' },
                    react_1.default.createElement("div", { className: 'save-analysis-label' }, "Existing analyses"),
                    react_1.default.createElement("div", { className: 'save-analysis-analysis-selection' },
                        props.savedAnalysisNames.length == 0 &&
                            react_1.default.createElement("p", null, "Save an analysis to have it appear here."),
                        props.savedAnalysisNames.map((savedAnalysisName) => {
                            return (react_1.default.createElement("div", { className: 'save-analysis-analysis-name', key: savedAnalysisName, onClick: () => { onChangeAnalysisName(savedAnalysisName); } }, savedAnalysisName));
                        }))))), buttons: react_1.default.createElement(react_1.Fragment, null,
            react_1.default.createElement("div", { className: 'modal-close-button modal-dual-button-left', onClick: () => { props.setModal({ type: Mito_1.ModalEnum.None }); } }, " Close "),
            react_1.default.createElement("div", { className: 'modal-action-button modal-dual-button-right', onClick: clickSave },
                " ",
                "Save")) }));
};
exports.default = SaveAnalysis;
//# sourceMappingURL=SaveAnalysisModal.js.map