"use strict";
// Copyright (c) Mito
// Distributed under the terms of the Modified BSD License.
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
const react_1 = __importDefault(require("react"));
const ag_grid_react_1 = require("ag-grid-react");
const ColumnHeader_1 = __importDefault(require("../components/ColumnHeader"));
const MITO_INDEX = 'mito_index';
// convert json formatted data frame into Ag-Grid data structure 
function buildGridData(sheet_json) {
    const gridData = [];
    const columns = sheet_json.columns;
    // iterate through the data frame to get each row
    for (let i = 0; i < sheet_json.data.length; i++) {
        const rowDict = {};
        // set the index column of the Ag-Grid
        rowDict[MITO_INDEX] = `${i + 1}`;
        // iterate through the column to get each element
        for (let j = 0; j < sheet_json.data[i].length; j++) {
            // create dict entry for row
            const rowDictKey = columns[j];
            rowDict[rowDictKey] = sheet_json.data[i][j];
        }
        gridData.push(rowDict);
    }
    return gridData;
}
exports.buildGridData = buildGridData;
// create columns from data frame columns
function buildGridColumns(df_columns, columnSpreadsheetCodeJSON, columnFiltersJSON, formulaBarValue, editingColumn, editingColumnCursorIndex, setEditingMode, setEditingFormula, setCursorIndex, setModal) {
    const gridColumns = [];
    // create index column
    gridColumns.push(react_1.default.createElement(ag_grid_react_1.AgGridColumn, { key: MITO_INDEX, headerName: '', resizable: true, field: MITO_INDEX, width: 10, lockPosition: true }));
    // iterate through columns of df_columns to create Ag-Grid columns
    df_columns.forEach((columnHeader) => {
        const headerName = columnHeader.toString();
        // only allow formula columns to be editable 
        const isEditable = columnSpreadsheetCodeJSON[headerName] !== '';
        /*
            if the column is the selected column:
                1. set the formula to the formula bar value to make sure that if the user has been editing the formula before ending
                cell editing mode, the most recent formula is used.
                2. set the cursor index to the last index the user was editing at
        */
        let columnFormula = columnSpreadsheetCodeJSON[headerName];
        let cursorIndex = columnFormula.length;
        if (headerName === editingColumn) {
            columnFormula = formulaBarValue;
            cursorIndex = editingColumnCursorIndex;
        }
        gridColumns.push(react_1.default.createElement(ag_grid_react_1.AgGridColumn, { key: headerName, field: headerName, headerName: headerName, headerComponentFramework: ColumnHeader_1.default, resizable: true, headerComponentParams: {
                setModal: setModal,
                displayName: headerName,
                filters: columnFiltersJSON[headerName].filters
            }, cellEditor: 'simpleEditor', cellEditorParams: {
                formula: columnFormula,
                setEditingMode: setEditingMode,
                setEditingFormula: setEditingFormula,
                setCursorIndex: setCursorIndex,
                cursorIndex: cursorIndex,
                columns: df_columns
            }, editable: isEditable, width: 125 }));
    });
    return gridColumns;
}
exports.buildGridColumns = buildGridColumns;
//# sourceMappingURL=gridData.js.map