"use strict";
// Copyright (c) Mito
// Distributed under the terms of the Modified BSD License.
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
const base_1 = require("@jupyter-widgets/base");
const version_1 = require("./version");
// Import the CSS
require("../css/widget.css");
// React
const react_1 = __importDefault(require("react"));
const react_dom_1 = __importDefault(require("react-dom"));
// Components
const Mito_1 = __importDefault(require("./components/Mito"));
// Logging
const analytics_node_1 = __importDefault(require("analytics-node"));
class ExampleModel extends base_1.DOMWidgetModel {
    // eslint-disable-next-line @typescript-eslint/explicit-module-boundary-types
    defaults() {
        return Object.assign(Object.assign({}, super.defaults()), { _model_name: ExampleModel.model_name, _model_module: ExampleModel.model_module, _model_module_version: ExampleModel.model_module_version, _view_name: ExampleModel.view_name, _view_module: ExampleModel.view_module, _view_module_version: ExampleModel.view_module_version, df_json: '' });
    }
}
exports.ExampleModel = ExampleModel;
ExampleModel.serializers = Object.assign({}, base_1.DOMWidgetModel.serializers);
ExampleModel.model_name = 'ExampleModel';
ExampleModel.model_module = version_1.MODULE_NAME;
ExampleModel.model_module_version = version_1.MODULE_VERSION;
ExampleModel.view_name = 'ExampleView'; // Set to null if no view
ExampleModel.view_module = version_1.MODULE_NAME; // Set to null if no view
ExampleModel.view_module_version = version_1.MODULE_VERSION;
const Mito_2 = require("./components/Mito");
class ExampleView extends base_1.DOMWidgetView {
    /*
      We override the DOMWidgetView constructor, so that we can
      create a logging instance for this view.
    */
    initialize(parameters) {
        super.initialize(parameters);
        // We get the user id from the client side
        window.user_id = this.model.get('user_id');
        // Write key taken from segment.com
        window.logger = new analytics_node_1.default('L4FqIZ3qB4C2FBitK4gUn073vv3lyWXm');
        // Identify the user
        window.logger.identify({ userId: window.user_id });
    }
    /*
      We override the sending message utilities, so that we can log all
      outgoing messages
    */
    send(msg) {
        var _a;
        // Log the out-going message
        (_a = window.logger) === null || _a === void 0 ? void 0 : _a.track({
            userId: window.user_id,
            event: 'js_sent_msg_log_event',
            properties: {
                event: msg
            }
        });
        super.send(msg);
    }
    render() {
        var _a, _b, _c;
        // Capture the send, to pass to the component
        const send = (msg) => {
            this.send(msg);
        };
        // TODO: there is a memory leak, in the case where
        // we rerender the component (e.g. we run the mito.sheet)
        // cell again. We need to clean up the component somehow!
        const model_id = this.model.model_id;
        react_dom_1.default.render(react_1.default.createElement(Mito_1.default, { dfNames: this.getDfNames(), sheetShapeArray: this.getSheetShapeArray(), sheetJSONArray: this.getSheetJSONArray(), columnSpreadsheetCodeJSONArray: this.getColumnSpreadsheetCodeJSONArray(), savedAnalysisNames: this.getSavedAnalysisNames(), columnFiltersArray: this.getColumnFilters(), columnTypeJSONArray: this.getColumnTypeJSONArray(), send: send, model_id: model_id, ref: (Mito) => {
                if (window.mitoMap === undefined) {
                    window.mitoMap = new Map();
                }
                window.mitoMap.set(model_id, Mito);
            } }), this.el);
        this.model.on('msg:custom', this.handleMessage, this);
        // TODO: make both of these only on the first render, not all of them... doh
        // Get df name from code block, and pass it to both the frontend and the backend
        (_a = window.commands) === null || _a === void 0 ? void 0 : _a.execute('get-df-names').then((dfNames) => {
            var _a, _b;
            // set the data frame name in the model state
            this.send({
                'event': 'update_event',
                'type': 'df_names_update',
                'df_names': dfNames
            });
            // set the data frame name in the widget
            (_b = (_a = window.mitoMap) === null || _a === void 0 ? void 0 : _a.get(this.model.model_id)) === null || _b === void 0 ? void 0 : _b.setState({ dfNames: dfNames });
        });
        // Get any previous analysis and send it back to the model!
        (_b = window.commands) === null || _b === void 0 ? void 0 : _b.execute('read-existing-analysis').then((analysisName) => {
            // If there is no previous analysis, we just ignore this step
            if (!analysisName)
                return;
            // And send it to the backend model
            this.send({
                'event': 'update_event',
                'type': 'use_existing_analysis_update',
                'analysis_name': analysisName
            });
            // We let the resulting update event update the backend!
        });
        // Log that this view was rendered
        (_c = window.logger) === null || _c === void 0 ? void 0 : _c.track({
            userId: window.user_id,
            event: 'sheet_view_creation_log_event',
            properties: {}
        });
    }
    getSheetJSONArray() {
        const sheetJSONArray = [];
        try {
            const modelSheetJSONArray = JSON.parse(this.model.get('sheet_json'));
            sheetJSONArray.push(...modelSheetJSONArray);
        }
        catch (e) {
            // Suppress error
            console.error(e);
        }
        return sheetJSONArray;
    }
    getSheetShapeArray() {
        const sheetShapeArray = [];
        try {
            const sheetShapeJSONArrayUnparsed = JSON.parse(this.model.get('df_shape_json'));
            sheetShapeArray.push(...sheetShapeJSONArrayUnparsed);
        }
        catch (e) {
            // Suppress error
            console.error(e);
        }
        return sheetShapeArray;
    }
    getColumnTypeJSONArray() {
        const columnTypeJSONArray = [];
        try {
            const modelColumnTypeJSONArray = JSON.parse(this.model.get('column_type_json'));
            columnTypeJSONArray.push(...modelColumnTypeJSONArray);
        }
        catch (e) {
            // Suppress error
            console.error(e);
        }
        return columnTypeJSONArray;
    }
    getSavedAnalysisNames() {
        const savedAnalysisNames = [];
        try {
            const savedAnalysisNamesJSON = JSON.parse(this.model.get('saved_analysis_names_json'));
            savedAnalysisNames.push(...savedAnalysisNamesJSON);
        }
        catch (e) {
            // Suppress error
            console.error(e);
        }
        return savedAnalysisNames;
    }
    getColumnSpreadsheetCodeJSONArray() {
        return JSON.parse(this.model.get('column_spreadsheet_code_json'));
    }
    getDfNames() {
        const dfNames = [];
        const unparsedDfNames = this.model.get('df_names_json');
        try {
            dfNames.push(...JSON.parse(unparsedDfNames)['df_names']);
            // And then we extend it to the length of the number of sheets,
            // as the dfNames sometimes aren't pulled correctly!
            const modelSheetJSONArray = this.getSheetJSONArray();
            for (let i = dfNames.length; i < modelSheetJSONArray.length; i++) {
                dfNames.push(`df${i + 1}`);
            }
        }
        catch (e) {
            // Suppress error
            console.error(e);
        }
        return dfNames;
    }
    getCodeJSON() {
        const codeJSON = {
            imports: '# No imports',
            code: ['# No code has been written yet!', 'pass']
        };
        const unparsedCodeJSON = this.model.get('code_json');
        try {
            codeJSON['imports'] = JSON.parse(unparsedCodeJSON)['imports'];
            codeJSON['code'] = JSON.parse(unparsedCodeJSON)['code'];
        }
        catch (e) {
            // Suppress error
            console.error(e);
        }
        return codeJSON;
    }
    getAnalysisName() {
        return this.model.get('analysis_name');
    }
    getColumnFilters() {
        return JSON.parse(this.model.get('column_filters_json'));
    }
    handleMessage(message) {
        /*
          This route handles the messages sent from the Python widget
        */
        var _a, _b, _c;
        // Log that we received this message
        (_a = window.logger) === null || _a === void 0 ? void 0 : _a.track({
            userId: window.user_id,
            event: 'js_recv_msg_log_event',
            properties: {
                event: message
            }
        });
        const model_id = this.model.model_id;
        const mito = (_b = window.mitoMap) === null || _b === void 0 ? void 0 : _b.get(model_id);
        if (mito === undefined) {
            console.error("Error: a message was received for a mito instance that does not exist!");
            return;
        }
        console.log("Got a message, ", message);
        if (message.event === 'update_sheet') {
            console.log("Updating sheet.");
            mito.setState({
                sheetJSONArray: this.getSheetJSONArray(),
                columnSpreadsheetCodeJSONArray: this.getColumnSpreadsheetCodeJSONArray(),
                dfNames: this.getDfNames(),
                sheetShapeArray: this.getSheetShapeArray(),
                savedAnalysisNames: this.getSavedAnalysisNames(),
                columnFiltersArray: this.getColumnFilters(),
                columnTypeJSONArray: this.getColumnTypeJSONArray()
            });
        }
        else if (message.event === 'update_code') {
            console.log('Updating code.');
            (_c = window.commands) === null || _c === void 0 ? void 0 : _c.execute('write-code-to-cell', {
                analysisName: this.getAnalysisName(),
                codeJSON: this.getCodeJSON()
            });
        }
        else if (message.event === 'edit_error') {
            console.log("Updating edit error.");
            mito.setState({
                modalInfo: { type: Mito_2.ModalEnum.Error },
                errorJSON: message
            });
        }
    }
}
exports.ExampleView = ExampleView;
//# sourceMappingURL=widget.js.map